@@ -39,10 +39,13 @@ use Getopt::Long qw(:config gnu_getopt);
 use File::Copy;
 use File::Basename;
 use Cwd;
-use Dpkg::Compression;
 use Dpkg::Vendor qw(get_current_vendor);
 use lib '/usr/share/devscripts';
+use Devscripts::Compression;
 use Devscripts::Debbugs;
+use POSIX qw(locale_h strftime);
+
+setlocale(LC_TIME, "C"); # so that strftime is locale independent
 
 # Predeclare functions
 sub fatal($);
@@ -54,6 +57,8 @@ my $modified_conf_msg;
 my %env;
 my $CHGLINE;  # used by the format O section at the end
 
+my $compression_re = compression_get_file_extension_regex();
+
 my $lpdc_broken;
 
 sub have_lpdc {
@@ -128,7 +133,7 @@ Options:
          Add a new changelog entry with version number specified
   -e, --edit
          Don't change version number or add a new changelog entry, just
-         update the changelog's stamp and open up an editor
+         opens an editor
   -r, --release
          Update the changelog timestamp. If the distribution is set to
          "UNRELEASED", change it to unstable (or another distribution as
@@ -989,8 +994,11 @@ if (@ARGV and ! $TEXT) {
 }
 
 # Get the date
-my $date_cmd = ($opt_tz ? "TZ=$opt_tz " : "") . "date -R";
-chomp(my $DATE=`$date_cmd`);
+my $DATE;
+{
+    local $ENV{TZ} = $opt_tz if $opt_tz;
+    $DATE = strftime "%a, %d %b %Y %T %z", localtime();
+}
 
 if ($opt_news && !$opt_i && !$opt_a) {
     if ($VERSION eq $changelog{'Version'} && !$opt_v && !$opt_l) {
@@ -1634,7 +1642,7 @@ if ((basename(cwd()) =~ m%^\Q$PACKAGE\E-\Q$UVERSION\E$%) &&
 	}
 	# And check whether a new orig tarball exists
 	my @origs = glob("../$PACKAGE\_$new_uversion.*");
-	my $num_origs = grep { /^..\/\Q$PACKAGE\E_\Q$new_uversion\E\.orig\.tar\.$compression_re_file_ext$/ } @origs;
+	my $num_origs = grep { /^..\/\Q$PACKAGE\E_\Q$new_uversion\E\.orig\.tar\.$compression_re$/ } @origs;
 	if ($num_origs == 0) {
 	    warn "$progname warning: no orig tarball found for the new version.\n";
 	}
