@@ -30,6 +30,7 @@ use Dpkg::IPC;
 use File::Basename;
 use File::Copy;
 use File::Temp qw/tempfile tempdir/;
+use List::Util qw/first/;
 use filetest 'access';
 use Getopt::Long qw(:config gnu_getopt);
 use lib '/usr/share/devscripts';
@@ -60,6 +61,7 @@ if ($@) {
     $haveSSL = 0;
 }
 my $havegpgv = (-x '/usr/bin/gpgv');
+my $havegpg = first { -x $_ } qw(/usr/bin/gpg2 /usr/bin/gpg);
 
 # Did we find any new upstream versions on our wanderings?
 our $found = 0;
@@ -702,6 +704,7 @@ sub process_watchline ($$$$$$)
     my $style='new';
     my $urlbase;
     my $headers = HTTP::Headers->new;
+    my ($keyring, $gpghome);
 
     # Comma-separated list of features that sites being queried might
     # want to be aware of
@@ -813,11 +816,24 @@ sub process_watchline ($$$$$$)
 
 	# Check validity of options
 	if (exists $options{'pgpsigurlmangle'}) {
-	    if (not (-r 'debian/upstream-signing-key.pgp')) {
-		uscan_warn "$progname warning: pgpsigurlmangle option exists, but debian/upstream-signing-key.pgp does not exist,\n  ignoring in $watchfile:\n  $line\n";
-		delete $options{'pgpsigurlmangle'};
-	    } elsif (! $havegpgv) {
-		uscan_warn "$progname warning: pgpsignurlmangle option exists, but you must have gpgv installed to verify\n  in $watchfile, skipping:\n  $line\n";
+	    if (! $havegpgv) {
+		uscan_warn "$progname warning: pgpsigurlmangle option exists, but you must have gpgv installed to verify\n  in $watchfile, skipping:\n  $line\n";
+		return 1;
+	    }
+	    $keyring = first { -r $_ } qw(debian/upstream/signing-key.pgp debian/upstream/signing-key.asc debian/upstream-signing-key.pgp);
+	    if ($keyring =~ m/\.asc$/) {
+		if (!$havegpg) {
+		    uscan_warn "$progname warning: $keyring is armored but gpg/gpg2 is not available to dearmor it\n  in $watchfile, skipping:\n $line\n";
+		    return 1;
+		}
+		# Need to convert an armored key to binary for use by gpgv
+		$gpghome = tempdir(CLEANUP => 1);
+		spawn(exec => [$havegpg, '--homedir', $gpghome, '--no-options', '-q', '--batch', '--no-default-keyring', '--import', $keyring],
+		      wait_child => 1);
+		$keyring = "$gpghome/pubring.gpg";
+	    }
+	    if (!defined $keyring) {
+		uscan_warn "$progname warning: pgpsigurlmangle option exists, but the upstream keyring does not exist\n  in $watchfile, skipping:\n  $line\n";
 		return 1;
 	    }
 	}
@@ -1350,66 +1366,64 @@ EOF
         print "Package directory '$destdir to store downloaded file is not existing\n";
         return 1;
     }
+    my $downloader = sub {
+	my ($url, $fname) = @_;
+	if ($url =~ m%^http(s)?://%) {
+	    if (defined($1) and !$haveSSL) {
+		uscan_die "$progname: you must have the liblwp-protocol-https-perl package installed\nto use https URLs\n";
+	    }
+	    # substitute HTML entities
+	    # Is anything else than "&amp;" required?  I doubt it.
+	    print STDERR "$progname debug: requesting URL $url\n" if $debug;
+	    my $headers = HTTP::Headers->new;
+	    $headers->header('Accept' => '*/*');
+	    $request = HTTP::Request->new('GET', $url, $headers);
+	    $response = $user_agent->request($request, $fname);
+	    if (! $response->is_success) {
+		if (defined $pkg_dir) {
+		    uscan_warn "$progname warning: In directory $pkg_dir, downloading\n  $url failed: " . $response->status_line . "\n";
+		} else {
+		    uscan_warn "$progname warning: Downloading\n $url failed:\n" . $response->status_line . "\n";
+		}
+		return 0;
+	    }
+	}
+	else {
+	    # FTP site
+	    if (exists $options{'pasv'}) {
+		$ENV{'FTP_PASSIVE'}=$options{'pasv'};
+	    }
+	    print STDERR "$progname debug: requesting URL $url\n" if $debug;
+	    $request = HTTP::Request->new('GET', "$url");
+	    $response = $user_agent->request($request, $fname);
+	    if (exists $options{'pasv'}) {
+		if (defined $passive) { $ENV{'FTP_PASSIVE'}=$passive; }
+		else { delete $ENV{'FTP_PASSIVE'}; }
+	    }
+	    if (! $response->is_success) {
+		if (defined $pkg_dir) {
+		    uscan_warn "$progname warning: In directory $pkg_dir, downloading\n  $url failed: " . $response->status_line . "\n";
+		} else {
+		    uscan_warn "$progname warning: Downloading\n $url failed:\n" . $response->status_line . "\n";
+		}
+		return 0;
+	    }
+	}
+	return 1;
+    };
     # Download newer package
-    if ($upstream_url =~ m%^http(s)?://%) {
-	if (defined($1) and !$haveSSL) {
-	    uscan_die "$progname: you must have the liblwp-protocol-https-perl package installed\nto use https URLs\n";
-	}
-	# substitute HTML entities
-	# Is anything else than "&amp;" required?  I doubt it.
-	print STDERR "$progname debug: requesting URL $upstream_url\n" if $debug;
-	my $headers = HTTP::Headers->new;
-	$headers->header('Accept' => '*/*');
-	$request = HTTP::Request->new('GET', $upstream_url, $headers);
-	$response = $user_agent->request($request, "$destdir/$newfile_base");
-	if (! $response->is_success) {
-	    if (defined $pkg_dir) {
-		uscan_warn "$progname warning: In directory $pkg_dir, downloading\n  $upstream_url failed: " . $response->status_line . "\n";
-	    } else {
-		uscan_warn "$progname warning: Downloading\n $upstream_url failed:\n" . $response->status_line . "\n";
-	    }
-	    return 1;
-	}
+    if (!$downloader->($upstream_url, "$destdir/$newfile_base")) {
+	return 1;
     }
-    else {
-	# FTP site
-	if (exists $options{'pasv'}) {
-	    $ENV{'FTP_PASSIVE'}=$options{'pasv'};
-	}
-	print STDERR "$progname debug: requesting URL $upstream_url\n" if $debug;
-	$request = HTTP::Request->new('GET', "$upstream_url");
-	$response = $user_agent->request($request, "$destdir/$newfile_base");
-	if (exists $options{'pasv'}) {
-	    if (defined $passive) { $ENV{'FTP_PASSIVE'}=$passive; }
-	    else { delete $ENV{'FTP_PASSIVE'}; }
-	}
-	if (! $response->is_success) {
-	    if (defined $pkg_dir) {
-		uscan_warn "$progname warning: In directory $pkg_dir, downloading\n  $upstream_url failed: " . $response->status_line . "\n";
-	    } else {
-		uscan_warn "$progname warning: Downloading\n $upstream_url failed:\n" . $response->status_line . "\n";
-	    }
-	    return 1;
-	}
-    }
-
     if (defined $pgpsig_url) {
 	print "-- Downloading OpenPGP signature for package as $newfile_base.pgp\n" if $verbose;
-	my $sigrequest = HTTP::Request->new('GET', "$pgpsig_url");
-	my $sigresponse = $user_agent->request($sigrequest, "$destdir/$newfile_base.pgp");
-
-	if (! $sigresponse->is_success) {
-	    if (defined $pkg_dir) {
-		uscan_warn "$progname warning: In directory $pkg_dir, downloading OpenPGP signature\n  $upstream_url failed: " . $sigresponse->status_line . "\n";
-	    } else {
-		uscan_warn "$progname warning: Downloading OpenPGP signature\n $pgpsig_url failed:\n" . $sigresponse->status_line . "\n";
-	    }
+	if (!$downloader->($pgpsig_url, "$destdir/$newfile_base.pgp")) {
 	    return 1;
 	}
 
 	print "-- Verifying OpenPGP signature $newfile_base.pgp for $newfile_base\n" if $verbose;
 	system('/usr/bin/gpgv', '--homedir', '/dev/null',
-	       '--keyring', 'debian/upstream-signing-key.pgp',
+	       '--keyring', $keyring,
 	       "$destdir/$newfile_base.pgp", "$destdir/$newfile_base") >> 8 == 0
 		 or uscan_die("$progname warning: OpenPGP signature did not verify.\n");
     }
@@ -1543,7 +1557,11 @@ EOF
 	    my $file_list;
 	    spawn(exec => ['find', $main_source_dir], wait_child => 1, to_string => \$file_list);
 	    my $nfiles_before = split /\n/, $file_list;
-	    foreach (grep { $_ } split /\s+/, $data->{"files-excluded"}) {
+	    # see the thread for details http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=635920#127
+	    my @excluded = ($data->{"files-excluded"} =~ /(?:\A|\G\s+)((?:\\.|[^\\\s])+)/g);
+	    # un-escape
+	    @excluded = map { s/\\(.)/$1/g; $_ } @excluded;
+	    foreach (@excluded) {
 		# delete trailing '/' because otherwise find -path will fail
 		s?/+$??;
 		# use rm -rf to enable deleting non-empty directories
@@ -2204,7 +2222,7 @@ sub get_main_source_dir($$$$) {
     $main_source_dir = $tempdir . '/' . $pkg . '-' . $newversion . $excludesuffix . '.orig';
     mkdir($main_source_dir) or uscan_die("Unable to create temporary source directory $main_source_dir\n");
     foreach my $file (@files) {
-	unless ($file =~ /^\.\.?/) {
+	unless ($file =~ /^\.\.?$/) {
 	    if ( -d "${tempdir}/$file" ) {
 		# HELP: why can't perl move not move directories????
 		system('mv', "${tempdir}/$file", $main_source_dir);
